import * as zlib from 'zlib';
import ISsnFileEntry from '../interfaces/ISsnFileEntry';
import decryptFile from './decryption/decryptFile';

class ByteReader {
  private dvArray: DataView[];
  private dvIndex = 0;
  private pos = 0;
  constructor(dvArray: DataView[], startDvIndex: number, offset: number) {
    this.dvArray = dvArray;
    this.dvIndex = startDvIndex;
    this.pos = offset;
  }
  public readByte() {
    const curByte = this.dvArray[this.dvIndex].getUint8(this.pos);
    this.pos += 1;
    if (this.pos >= this.dvArray[this.dvIndex].byteLength) {
      this.pos = 0;
      this.dvIndex += 1;
      if (this.dvIndex >= this.dvArray.length) { throw new Error('Tried to read beyond DataView boundary in extractFile'); }
    }
    return curByte;
  }
  public seek(num: number) {
    this.pos += num;
    if (this.pos >= this.dvArray[this.dvIndex].byteLength) {
      this.pos -= this.dvArray[this.dvIndex].byteLength;
      this.dvIndex += 1;
      if (this.dvIndex >= this.dvArray.length) { throw new Error('Tried to read beyond DataView boundary in extractFile'); }
    }
  }
  public extractDv(length: number) {
    const dv = new DataView(new ArrayBuffer(length));
    for (let i = 0; i < length; i += 1) {
        dv.setUint8(i, this.readByte()); //TODO: refactor this so it is more optimized
    }
    return dv;
  }
}

/** Extracts the given file from the given DataView array and returns it as an ArrayBuffer.
 * Will throw an error when end of final DataView is reached.
 */
export default async function extractFile(file: ISsnFileEntry, dvArray: DataView[]): Promise<ArrayBuffer> {
  //use ByteReader for reading a uint8 and seeking forward across DataView boundaries
  const byteReader = new ByteReader(dvArray, file.diskNumberStart, file.offset);

  if (byteReader.readByte() !== 0x50 || byteReader.readByte() !== 0x4B || byteReader.readByte() !== 0x03 || byteReader.readByte() !== 0x04) {
        throw new Error('Local file header had wrong magic');
  }
  byteReader.seek(22);
  const localFilenameSize = byteReader.readByte() + (byteReader.readByte() << 8);
  const localExtraSize = byteReader.readByte() + (byteReader.readByte() << 8);
  byteReader.seek(localFilenameSize + localExtraSize);

  let dvFinal = byteReader.extractDv(file.comprSize);

  //decrypt file if necessary
  if (file.decryptionKeys !== undefined) {
    dvFinal = decryptFile(dvFinal, file.comprSize, file.decryptionKeys);
  }

  //uncompress file
  const uncompressedBuffer: Buffer = await new Promise((resolve) => {
    zlib.inflateRaw(dvFinal, (error, result) => {
      resolve(result);
    });
  }) as Buffer;
  return uncompressedBuffer.buffer as ArrayBuffer;
}
