import { TextDecoder } from 'util';
import getUrlContents from '../getUrlContents';
import { Product } from '../interfaces/ISettings';
import verifyProductName from '../ssn/verifyProductName';
import extractFile from './extractFile';
import readSsnFile from './readSsnFile';

const Decoder = new TextDecoder('utf-8');

export default async function getPatchmanifest(product: Product): Promise<string> {
  //Verify function arguments
  if (!verifyProductName(product)) {
    throw new Error(`"${product}" is not a valid product.`);
  }

  //Download .patchmanifest file
  const ssnFile = await getUrlContents({ host: 'manifest.swtor.com', path: `/patch/${product}.patchmanifest` });

  //Parse .patchmanifest file
  const fileEntries = readSsnFile(ssnFile);

  if (fileEntries.length !== 1) {
    throw new Error(`Expected .patchmanifest to contain 1 file but it had "${fileEntries.length}" files.`);
  }

  const firstFile = fileEntries[0];
  if (firstFile.name !== 'manifest.xml') {
    throw new Error(`Expected .patchmanifest to contain a file called manifest.xml but it is called "${firstFile.name}".`);
  }

  //Extract manifest.xml file
  const patchmanifestFile = await extractFile(firstFile, [new DataView(ssnFile)]);
  const patchmanifestXml = Decoder.decode(patchmanifestFile);

  return patchmanifestXml;
}
