import ISolid from '../interfaces/ISolidFile';

/** Verifies metafile.solid for correctness. */
export default function verifySolidpkg(file: ISolid, { product, from, to }: {product: string, from: number, to: number}) {
  if (typeof file['creation date'] !== 'number') {
    throw new Error(`Expected creation date to be a number but it was "${file['creation date']}".`);
  }
  if (file.announce !== 'http://Tracker22.fda5b1cf-eac6-402b-9ebf-17bc381314aa.automated.ssntracker.int:80/') {
    throw new Error(`Expected announce URL but it was "${file.announce}".`);
  }
  if (file.title !== `${product}: ${from}to${to}`) {
    throw new Error(`Expected title "${product}: ${from}to${to}" but it was "${file.title}".`);
  }
  if (file.networkgroupid !== 16097) {
    throw new Error(`Expected networkgroupid 16097 but it was "${file.networkgroupid}".`);
  }
  if (file.reliable !== `http://cdn-patch.swtor.com/patch/${product}/${product}_${from}to${to}/`) {
    throw new Error(`Expected reliable URL but it was "${file.reliable}".`);
  }
  if (file['reliable-id'] !== '0') {
    throw new Error(`Expected reliable-id to be "0" but it was "${file['reliable-id']}".`);
  }

  if (file.info === undefined) {
    throw new Error(`Expected info field but it was missing.`);
  }

  //TODO: read file.info.files

  if (file.info['piece length'] !== 4194304) {
    throw new Error(`Expected piece length to be "4194304" but it was "${file.info['piece length']}".`);
  }
  if (typeof file.info.pieces !== 'string') {
    throw new Error(`Expected pieces to be a string but it was "${file.info.pieces}".`);
  }
  if (file.info.private !== 0) {
    throw new Error(`Expected file to not be private ("0") but it was "${file.info.private}".`);
  }
  if (file.info.closed !== 1) {
    throw new Error(`Expected file to be closed ("1") but it was "${file.info.closed}".`);
  }
  if (typeof file.info.uniqueid !== 'string') {
    throw new Error(`Expected uniqueid to be a string but it was "${file.info.uniqueid}".`);
  }
}
